# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cosmos/distribution/v1beta1/genesis.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2
from cosmos.distribution.v1beta1 import distribution_pb2 as cosmos_dot_distribution_dot_v1beta1_dot_distribution__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='cosmos/distribution/v1beta1/genesis.proto',
  package='cosmos.distribution.v1beta1',
  syntax='proto3',
  serialized_options=b'Z1github.com/cosmos/cosmos-sdk/x/distribution/types\250\342\036\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n)cosmos/distribution/v1beta1/genesis.proto\x12\x1b\x63osmos.distribution.v1beta1\x1a\x14gogoproto/gogo.proto\x1a\x1e\x63osmos/base/v1beta1/coin.proto\x1a.cosmos/distribution/v1beta1/distribution.proto\"\x91\x01\n\x15\x44\x65legatorWithdrawInfo\x12\x37\n\x11\x64\x65legator_address\x18\x01 \x01(\tB\x1c\xf2\xde\x1f\x18yaml:\"delegator_address\"\x12\x35\n\x10withdraw_address\x18\x02 \x01(\tB\x1b\xf2\xde\x1f\x17yaml:\"withdraw_address\":\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"\xf5\x01\n!ValidatorOutstandingRewardsRecord\x12\x37\n\x11validator_address\x18\x01 \x01(\tB\x1c\xf2\xde\x1f\x18yaml:\"validator_address\"\x12\x8c\x01\n\x13outstanding_rewards\x18\x02 \x03(\x0b\x32\x1c.cosmos.base.v1beta1.DecCoinBQ\xaa\xdf\x1f+github.com/cosmos/cosmos-sdk/types.DecCoins\xc8\xde\x1f\x00\xf2\xde\x1f\x1ayaml:\"outstanding_rewards\":\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"\xd7\x01\n$ValidatorAccumulatedCommissionRecord\x12\x37\n\x11validator_address\x18\x01 \x01(\tB\x1c\xf2\xde\x1f\x18yaml:\"validator_address\"\x12l\n\x0b\x61\x63\x63umulated\x18\x02 \x01(\x0b\x32;.cosmos.distribution.v1beta1.ValidatorAccumulatedCommissionB\x1a\xc8\xde\x1f\x00\xf2\xde\x1f\x12yaml:\"accumulated\":\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"\xd7\x01\n ValidatorHistoricalRewardsRecord\x12\x37\n\x11validator_address\x18\x01 \x01(\tB\x1c\xf2\xde\x1f\x18yaml:\"validator_address\"\x12\x0e\n\x06period\x18\x02 \x01(\x04\x12`\n\x07rewards\x18\x03 \x01(\x0b\x32\x37.cosmos.distribution.v1beta1.ValidatorHistoricalRewardsB\x16\xc8\xde\x1f\x00\xf2\xde\x1f\x0eyaml:\"rewards\":\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"\xc1\x01\n\x1dValidatorCurrentRewardsRecord\x12\x37\n\x11validator_address\x18\x01 \x01(\tB\x1c\xf2\xde\x1f\x18yaml:\"validator_address\"\x12]\n\x07rewards\x18\x02 \x01(\x0b\x32\x34.cosmos.distribution.v1beta1.ValidatorCurrentRewardsB\x16\xc8\xde\x1f\x00\xf2\xde\x1f\x0eyaml:\"rewards\":\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"\x82\x02\n\x1b\x44\x65legatorStartingInfoRecord\x12\x37\n\x11\x64\x65legator_address\x18\x01 \x01(\tB\x1c\xf2\xde\x1f\x18yaml:\"delegator_address\"\x12\x37\n\x11validator_address\x18\x02 \x01(\tB\x1c\xf2\xde\x1f\x18yaml:\"validator_address\"\x12g\n\rstarting_info\x18\x03 \x01(\x0b\x32\x32.cosmos.distribution.v1beta1.DelegatorStartingInfoB\x1c\xc8\xde\x1f\x00\xf2\xde\x1f\x14yaml:\"starting_info\":\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"\xe5\x01\n\x19ValidatorSlashEventRecord\x12\x37\n\x11validator_address\x18\x01 \x01(\tB\x1c\xf2\xde\x1f\x18yaml:\"validator_address\"\x12\x0e\n\x06height\x18\x02 \x01(\x04\x12\x0e\n\x06period\x18\x03 \x01(\x04\x12\x65\n\x15validator_slash_event\x18\x04 \x01(\x0b\x32\x30.cosmos.distribution.v1beta1.ValidatorSlashEventB\x14\xc8\xde\x1f\x00\xf2\xde\x1f\x0cyaml:\"event\":\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"\xb1\t\n\x0cGenesisState\x12J\n\x06params\x18\x01 \x01(\x0b\x32#.cosmos.distribution.v1beta1.ParamsB\x15\xc8\xde\x1f\x00\xf2\xde\x1f\ryaml:\"params\"\x12O\n\x08\x66\x65\x65_pool\x18\x02 \x01(\x0b\x32$.cosmos.distribution.v1beta1.FeePoolB\x17\xc8\xde\x1f\x00\xf2\xde\x1f\x0fyaml:\"fee_pool\"\x12}\n\x18\x64\x65legator_withdraw_infos\x18\x03 \x03(\x0b\x32\x32.cosmos.distribution.v1beta1.DelegatorWithdrawInfoB\'\xc8\xde\x1f\x00\xf2\xde\x1f\x1fyaml:\"delegator_withdraw_infos\"\x12\x37\n\x11previous_proposer\x18\x04 \x01(\tB\x1c\xf2\xde\x1f\x18yaml:\"previous_proposer\"\x12\x7f\n\x13outstanding_rewards\x18\x05 \x03(\x0b\x32>.cosmos.distribution.v1beta1.ValidatorOutstandingRewardsRecordB\"\xc8\xde\x1f\x00\xf2\xde\x1f\x1ayaml:\"outstanding_rewards\"\x12\x9e\x01\n!validator_accumulated_commissions\x18\x06 \x03(\x0b\x32\x41.cosmos.distribution.v1beta1.ValidatorAccumulatedCommissionRecordB0\xc8\xde\x1f\x00\xf2\xde\x1f(yaml:\"validator_accumulated_commissions\"\x12\x90\x01\n\x1cvalidator_historical_rewards\x18\x07 \x03(\x0b\x32=.cosmos.distribution.v1beta1.ValidatorHistoricalRewardsRecordB+\xc8\xde\x1f\x00\xf2\xde\x1f#yaml:\"validator_historical_rewards\"\x12\x87\x01\n\x19validator_current_rewards\x18\x08 \x03(\x0b\x32:.cosmos.distribution.v1beta1.ValidatorCurrentRewardsRecordB(\xc8\xde\x1f\x00\xf2\xde\x1f yaml:\"validator_current_rewards\"\x12\x83\x01\n\x18\x64\x65legator_starting_infos\x18\t \x03(\x0b\x32\x38.cosmos.distribution.v1beta1.DelegatorStartingInfoRecordB\'\xc8\xde\x1f\x00\xf2\xde\x1f\x1fyaml:\"delegator_starting_infos\"\x12}\n\x16validator_slash_events\x18\n \x03(\x0b\x32\x36.cosmos.distribution.v1beta1.ValidatorSlashEventRecordB%\xc8\xde\x1f\x00\xf2\xde\x1f\x1dyaml:\"validator_slash_events\":\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\x42\x37Z1github.com/cosmos/cosmos-sdk/x/distribution/types\xa8\xe2\x1e\x01\x62\x06proto3'
  ,
  dependencies=[gogoproto_dot_gogo__pb2.DESCRIPTOR,cosmos_dot_base_dot_v1beta1_dot_coin__pb2.DESCRIPTOR,cosmos_dot_distribution_dot_v1beta1_dot_distribution__pb2.DESCRIPTOR,])




_DELEGATORWITHDRAWINFO = _descriptor.Descriptor(
  name='DelegatorWithdrawInfo',
  full_name='cosmos.distribution.v1beta1.DelegatorWithdrawInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='delegator_address', full_name='cosmos.distribution.v1beta1.DelegatorWithdrawInfo.delegator_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\030yaml:\"delegator_address\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='withdraw_address', full_name='cosmos.distribution.v1beta1.DelegatorWithdrawInfo.withdraw_address', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\027yaml:\"withdraw_address\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\350\240\037\000\210\240\037\000',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=177,
  serialized_end=322,
)


_VALIDATOROUTSTANDINGREWARDSRECORD = _descriptor.Descriptor(
  name='ValidatorOutstandingRewardsRecord',
  full_name='cosmos.distribution.v1beta1.ValidatorOutstandingRewardsRecord',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='validator_address', full_name='cosmos.distribution.v1beta1.ValidatorOutstandingRewardsRecord.validator_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\030yaml:\"validator_address\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='outstanding_rewards', full_name='cosmos.distribution.v1beta1.ValidatorOutstandingRewardsRecord.outstanding_rewards', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\252\337\037+github.com/cosmos/cosmos-sdk/types.DecCoins\310\336\037\000\362\336\037\032yaml:\"outstanding_rewards\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\350\240\037\000\210\240\037\000',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=325,
  serialized_end=570,
)


_VALIDATORACCUMULATEDCOMMISSIONRECORD = _descriptor.Descriptor(
  name='ValidatorAccumulatedCommissionRecord',
  full_name='cosmos.distribution.v1beta1.ValidatorAccumulatedCommissionRecord',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='validator_address', full_name='cosmos.distribution.v1beta1.ValidatorAccumulatedCommissionRecord.validator_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\030yaml:\"validator_address\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='accumulated', full_name='cosmos.distribution.v1beta1.ValidatorAccumulatedCommissionRecord.accumulated', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000\362\336\037\022yaml:\"accumulated\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\350\240\037\000\210\240\037\000',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=573,
  serialized_end=788,
)


_VALIDATORHISTORICALREWARDSRECORD = _descriptor.Descriptor(
  name='ValidatorHistoricalRewardsRecord',
  full_name='cosmos.distribution.v1beta1.ValidatorHistoricalRewardsRecord',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='validator_address', full_name='cosmos.distribution.v1beta1.ValidatorHistoricalRewardsRecord.validator_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\030yaml:\"validator_address\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='period', full_name='cosmos.distribution.v1beta1.ValidatorHistoricalRewardsRecord.period', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rewards', full_name='cosmos.distribution.v1beta1.ValidatorHistoricalRewardsRecord.rewards', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000\362\336\037\016yaml:\"rewards\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\350\240\037\000\210\240\037\000',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=791,
  serialized_end=1006,
)


_VALIDATORCURRENTREWARDSRECORD = _descriptor.Descriptor(
  name='ValidatorCurrentRewardsRecord',
  full_name='cosmos.distribution.v1beta1.ValidatorCurrentRewardsRecord',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='validator_address', full_name='cosmos.distribution.v1beta1.ValidatorCurrentRewardsRecord.validator_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\030yaml:\"validator_address\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rewards', full_name='cosmos.distribution.v1beta1.ValidatorCurrentRewardsRecord.rewards', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000\362\336\037\016yaml:\"rewards\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\350\240\037\000\210\240\037\000',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1009,
  serialized_end=1202,
)


_DELEGATORSTARTINGINFORECORD = _descriptor.Descriptor(
  name='DelegatorStartingInfoRecord',
  full_name='cosmos.distribution.v1beta1.DelegatorStartingInfoRecord',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='delegator_address', full_name='cosmos.distribution.v1beta1.DelegatorStartingInfoRecord.delegator_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\030yaml:\"delegator_address\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='validator_address', full_name='cosmos.distribution.v1beta1.DelegatorStartingInfoRecord.validator_address', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\030yaml:\"validator_address\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='starting_info', full_name='cosmos.distribution.v1beta1.DelegatorStartingInfoRecord.starting_info', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000\362\336\037\024yaml:\"starting_info\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\350\240\037\000\210\240\037\000',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1205,
  serialized_end=1463,
)


_VALIDATORSLASHEVENTRECORD = _descriptor.Descriptor(
  name='ValidatorSlashEventRecord',
  full_name='cosmos.distribution.v1beta1.ValidatorSlashEventRecord',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='validator_address', full_name='cosmos.distribution.v1beta1.ValidatorSlashEventRecord.validator_address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\030yaml:\"validator_address\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='height', full_name='cosmos.distribution.v1beta1.ValidatorSlashEventRecord.height', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='period', full_name='cosmos.distribution.v1beta1.ValidatorSlashEventRecord.period', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='validator_slash_event', full_name='cosmos.distribution.v1beta1.ValidatorSlashEventRecord.validator_slash_event', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000\362\336\037\014yaml:\"event\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\350\240\037\000\210\240\037\000',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1466,
  serialized_end=1695,
)


_GENESISSTATE = _descriptor.Descriptor(
  name='GenesisState',
  full_name='cosmos.distribution.v1beta1.GenesisState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='params', full_name='cosmos.distribution.v1beta1.GenesisState.params', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000\362\336\037\ryaml:\"params\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fee_pool', full_name='cosmos.distribution.v1beta1.GenesisState.fee_pool', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000\362\336\037\017yaml:\"fee_pool\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='delegator_withdraw_infos', full_name='cosmos.distribution.v1beta1.GenesisState.delegator_withdraw_infos', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000\362\336\037\037yaml:\"delegator_withdraw_infos\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='previous_proposer', full_name='cosmos.distribution.v1beta1.GenesisState.previous_proposer', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\030yaml:\"previous_proposer\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='outstanding_rewards', full_name='cosmos.distribution.v1beta1.GenesisState.outstanding_rewards', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000\362\336\037\032yaml:\"outstanding_rewards\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='validator_accumulated_commissions', full_name='cosmos.distribution.v1beta1.GenesisState.validator_accumulated_commissions', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000\362\336\037(yaml:\"validator_accumulated_commissions\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='validator_historical_rewards', full_name='cosmos.distribution.v1beta1.GenesisState.validator_historical_rewards', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000\362\336\037#yaml:\"validator_historical_rewards\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='validator_current_rewards', full_name='cosmos.distribution.v1beta1.GenesisState.validator_current_rewards', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000\362\336\037 yaml:\"validator_current_rewards\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='delegator_starting_infos', full_name='cosmos.distribution.v1beta1.GenesisState.delegator_starting_infos', index=8,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000\362\336\037\037yaml:\"delegator_starting_infos\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='validator_slash_events', full_name='cosmos.distribution.v1beta1.GenesisState.validator_slash_events', index=9,
      number=10, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000\362\336\037\035yaml:\"validator_slash_events\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\350\240\037\000\210\240\037\000',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1698,
  serialized_end=2899,
)

_VALIDATOROUTSTANDINGREWARDSRECORD.fields_by_name['outstanding_rewards'].message_type = cosmos_dot_base_dot_v1beta1_dot_coin__pb2._DECCOIN
_VALIDATORACCUMULATEDCOMMISSIONRECORD.fields_by_name['accumulated'].message_type = cosmos_dot_distribution_dot_v1beta1_dot_distribution__pb2._VALIDATORACCUMULATEDCOMMISSION
_VALIDATORHISTORICALREWARDSRECORD.fields_by_name['rewards'].message_type = cosmos_dot_distribution_dot_v1beta1_dot_distribution__pb2._VALIDATORHISTORICALREWARDS
_VALIDATORCURRENTREWARDSRECORD.fields_by_name['rewards'].message_type = cosmos_dot_distribution_dot_v1beta1_dot_distribution__pb2._VALIDATORCURRENTREWARDS
_DELEGATORSTARTINGINFORECORD.fields_by_name['starting_info'].message_type = cosmos_dot_distribution_dot_v1beta1_dot_distribution__pb2._DELEGATORSTARTINGINFO
_VALIDATORSLASHEVENTRECORD.fields_by_name['validator_slash_event'].message_type = cosmos_dot_distribution_dot_v1beta1_dot_distribution__pb2._VALIDATORSLASHEVENT
_GENESISSTATE.fields_by_name['params'].message_type = cosmos_dot_distribution_dot_v1beta1_dot_distribution__pb2._PARAMS
_GENESISSTATE.fields_by_name['fee_pool'].message_type = cosmos_dot_distribution_dot_v1beta1_dot_distribution__pb2._FEEPOOL
_GENESISSTATE.fields_by_name['delegator_withdraw_infos'].message_type = _DELEGATORWITHDRAWINFO
_GENESISSTATE.fields_by_name['outstanding_rewards'].message_type = _VALIDATOROUTSTANDINGREWARDSRECORD
_GENESISSTATE.fields_by_name['validator_accumulated_commissions'].message_type = _VALIDATORACCUMULATEDCOMMISSIONRECORD
_GENESISSTATE.fields_by_name['validator_historical_rewards'].message_type = _VALIDATORHISTORICALREWARDSRECORD
_GENESISSTATE.fields_by_name['validator_current_rewards'].message_type = _VALIDATORCURRENTREWARDSRECORD
_GENESISSTATE.fields_by_name['delegator_starting_infos'].message_type = _DELEGATORSTARTINGINFORECORD
_GENESISSTATE.fields_by_name['validator_slash_events'].message_type = _VALIDATORSLASHEVENTRECORD
DESCRIPTOR.message_types_by_name['DelegatorWithdrawInfo'] = _DELEGATORWITHDRAWINFO
DESCRIPTOR.message_types_by_name['ValidatorOutstandingRewardsRecord'] = _VALIDATOROUTSTANDINGREWARDSRECORD
DESCRIPTOR.message_types_by_name['ValidatorAccumulatedCommissionRecord'] = _VALIDATORACCUMULATEDCOMMISSIONRECORD
DESCRIPTOR.message_types_by_name['ValidatorHistoricalRewardsRecord'] = _VALIDATORHISTORICALREWARDSRECORD
DESCRIPTOR.message_types_by_name['ValidatorCurrentRewardsRecord'] = _VALIDATORCURRENTREWARDSRECORD
DESCRIPTOR.message_types_by_name['DelegatorStartingInfoRecord'] = _DELEGATORSTARTINGINFORECORD
DESCRIPTOR.message_types_by_name['ValidatorSlashEventRecord'] = _VALIDATORSLASHEVENTRECORD
DESCRIPTOR.message_types_by_name['GenesisState'] = _GENESISSTATE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DelegatorWithdrawInfo = _reflection.GeneratedProtocolMessageType('DelegatorWithdrawInfo', (_message.Message,), {
  'DESCRIPTOR' : _DELEGATORWITHDRAWINFO,
  '__module__' : 'cosmos.distribution.v1beta1.genesis_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.DelegatorWithdrawInfo)
  })
_sym_db.RegisterMessage(DelegatorWithdrawInfo)

ValidatorOutstandingRewardsRecord = _reflection.GeneratedProtocolMessageType('ValidatorOutstandingRewardsRecord', (_message.Message,), {
  'DESCRIPTOR' : _VALIDATOROUTSTANDINGREWARDSRECORD,
  '__module__' : 'cosmos.distribution.v1beta1.genesis_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.ValidatorOutstandingRewardsRecord)
  })
_sym_db.RegisterMessage(ValidatorOutstandingRewardsRecord)

ValidatorAccumulatedCommissionRecord = _reflection.GeneratedProtocolMessageType('ValidatorAccumulatedCommissionRecord', (_message.Message,), {
  'DESCRIPTOR' : _VALIDATORACCUMULATEDCOMMISSIONRECORD,
  '__module__' : 'cosmos.distribution.v1beta1.genesis_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.ValidatorAccumulatedCommissionRecord)
  })
_sym_db.RegisterMessage(ValidatorAccumulatedCommissionRecord)

ValidatorHistoricalRewardsRecord = _reflection.GeneratedProtocolMessageType('ValidatorHistoricalRewardsRecord', (_message.Message,), {
  'DESCRIPTOR' : _VALIDATORHISTORICALREWARDSRECORD,
  '__module__' : 'cosmos.distribution.v1beta1.genesis_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.ValidatorHistoricalRewardsRecord)
  })
_sym_db.RegisterMessage(ValidatorHistoricalRewardsRecord)

ValidatorCurrentRewardsRecord = _reflection.GeneratedProtocolMessageType('ValidatorCurrentRewardsRecord', (_message.Message,), {
  'DESCRIPTOR' : _VALIDATORCURRENTREWARDSRECORD,
  '__module__' : 'cosmos.distribution.v1beta1.genesis_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.ValidatorCurrentRewardsRecord)
  })
_sym_db.RegisterMessage(ValidatorCurrentRewardsRecord)

DelegatorStartingInfoRecord = _reflection.GeneratedProtocolMessageType('DelegatorStartingInfoRecord', (_message.Message,), {
  'DESCRIPTOR' : _DELEGATORSTARTINGINFORECORD,
  '__module__' : 'cosmos.distribution.v1beta1.genesis_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.DelegatorStartingInfoRecord)
  })
_sym_db.RegisterMessage(DelegatorStartingInfoRecord)

ValidatorSlashEventRecord = _reflection.GeneratedProtocolMessageType('ValidatorSlashEventRecord', (_message.Message,), {
  'DESCRIPTOR' : _VALIDATORSLASHEVENTRECORD,
  '__module__' : 'cosmos.distribution.v1beta1.genesis_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.ValidatorSlashEventRecord)
  })
_sym_db.RegisterMessage(ValidatorSlashEventRecord)

GenesisState = _reflection.GeneratedProtocolMessageType('GenesisState', (_message.Message,), {
  'DESCRIPTOR' : _GENESISSTATE,
  '__module__' : 'cosmos.distribution.v1beta1.genesis_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.GenesisState)
  })
_sym_db.RegisterMessage(GenesisState)


DESCRIPTOR._options = None
_DELEGATORWITHDRAWINFO.fields_by_name['delegator_address']._options = None
_DELEGATORWITHDRAWINFO.fields_by_name['withdraw_address']._options = None
_DELEGATORWITHDRAWINFO._options = None
_VALIDATOROUTSTANDINGREWARDSRECORD.fields_by_name['validator_address']._options = None
_VALIDATOROUTSTANDINGREWARDSRECORD.fields_by_name['outstanding_rewards']._options = None
_VALIDATOROUTSTANDINGREWARDSRECORD._options = None
_VALIDATORACCUMULATEDCOMMISSIONRECORD.fields_by_name['validator_address']._options = None
_VALIDATORACCUMULATEDCOMMISSIONRECORD.fields_by_name['accumulated']._options = None
_VALIDATORACCUMULATEDCOMMISSIONRECORD._options = None
_VALIDATORHISTORICALREWARDSRECORD.fields_by_name['validator_address']._options = None
_VALIDATORHISTORICALREWARDSRECORD.fields_by_name['rewards']._options = None
_VALIDATORHISTORICALREWARDSRECORD._options = None
_VALIDATORCURRENTREWARDSRECORD.fields_by_name['validator_address']._options = None
_VALIDATORCURRENTREWARDSRECORD.fields_by_name['rewards']._options = None
_VALIDATORCURRENTREWARDSRECORD._options = None
_DELEGATORSTARTINGINFORECORD.fields_by_name['delegator_address']._options = None
_DELEGATORSTARTINGINFORECORD.fields_by_name['validator_address']._options = None
_DELEGATORSTARTINGINFORECORD.fields_by_name['starting_info']._options = None
_DELEGATORSTARTINGINFORECORD._options = None
_VALIDATORSLASHEVENTRECORD.fields_by_name['validator_address']._options = None
_VALIDATORSLASHEVENTRECORD.fields_by_name['validator_slash_event']._options = None
_VALIDATORSLASHEVENTRECORD._options = None
_GENESISSTATE.fields_by_name['params']._options = None
_GENESISSTATE.fields_by_name['fee_pool']._options = None
_GENESISSTATE.fields_by_name['delegator_withdraw_infos']._options = None
_GENESISSTATE.fields_by_name['previous_proposer']._options = None
_GENESISSTATE.fields_by_name['outstanding_rewards']._options = None
_GENESISSTATE.fields_by_name['validator_accumulated_commissions']._options = None
_GENESISSTATE.fields_by_name['validator_historical_rewards']._options = None
_GENESISSTATE.fields_by_name['validator_current_rewards']._options = None
_GENESISSTATE.fields_by_name['delegator_starting_infos']._options = None
_GENESISSTATE.fields_by_name['validator_slash_events']._options = None
_GENESISSTATE._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2022 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Tests for REST implementation of Slashing."""
import json
from typing import Dict, Tuple
from unittest import TestCase

from google.protobuf.json_format import ParseDict

from cosmpy.protos.cosmos.slashing.v1beta1.query_pb2 import (
    QueryParamsResponse,
    QuerySigningInfoRequest,
    QuerySigningInfoResponse,
    QuerySigningInfosRequest,
    QuerySigningInfosResponse,
)
from cosmpy.slashing.rest_client import SlashingRestClient
from tests.helpers import MockRestClient


class SlashingRestClientTestCase(TestCase):
    """Test case for SlashingRestClient class."""

    REST_CLIENT = SlashingRestClient

    def make_clients(
        self, response_content: Dict
    ) -> Tuple[MockRestClient, SlashingRestClient]:
        """
        Make  mock client and rest client api for specific content.

        :param response_content: dict
        :return: rest client instance
        """
        mock_client = MockRestClient(json.dumps(response_content).encode("utf-8"))
        rest_client = self.REST_CLIENT(mock_client)
        return mock_client, rest_client

    def test_Params(self):
        """Test Params method."""
        content = {
            "params": {
                "signed_blocks_window": "12",
                "min_signed_per_window": "12",
                "downtime_jail_duration": "12s",
                "slash_fraction_double_sign": "12",
                "slash_fraction_downtime": "12",
            }
        }
        mock_client, slashing = self.make_clients(content)
        expected_response = ParseDict(content, QueryParamsResponse())

        assert slashing.Params() == expected_response
        assert mock_client.last_base_url == "/cosmos/slashing/v1beta1/params"

    def test_SigningInfo(self):
        """Test SigningInfo method."""
        content = {
            "val_signing_info": {
                "address": "string",
                "start_height": "1",
                "index_offset": "1",
                "jailed_until": "2022-03-22T11:14:55.544Z",
                "tombstoned": True,
                "missed_blocks_counter": "1",
            }
        }
        mock_client, slashing = self.make_clients(content)
        expected_response = ParseDict(content, QuerySigningInfoResponse())

        assert (
            slashing.SigningInfo(QuerySigningInfoRequest(cons_address="some_addr"))
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/cosmos/slashing/v1beta1/signing_infos/some_addr"
        )

    def test_SigningInfos(self):
        """Test SigningInfos method."""
        content = {
            "info": [
                {
                    "address": "string",
                    "start_height": "1",
                    "index_offset": "1",
                    "jailed_until": "2022-03-22T11:15:53.940Z",
                    "tombstoned": True,
                    "missed_blocks_counter": "1",
                }
            ],
            "pagination": {"next_key": "string", "total": "1"},
        }
        mock_client, slashing = self.make_clients(content)
        expected_response = ParseDict(content, QuerySigningInfosResponse())

        assert slashing.SigningInfos(QuerySigningInfosRequest()) == expected_response
        assert mock_client.last_base_url == "/cosmos/slashing/v1beta1/signing_infos"



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Tests for REST implementation of Staking."""

import json
from unittest import TestCase

from google.protobuf.json_format import ParseDict

from cosmpy.params.rest_client import ParamsRestClient
from cosmpy.protos.cosmos.params.v1beta1.query_pb2 import (
    QueryParamsRequest,
    QueryParamsResponse,
)
from tests.helpers import MockRestClient


class ParamsRestClientTestCase(TestCase):
    """Test case for ParamsRestClient class."""

    @staticmethod
    def test_Validators():
        """Test Validators method."""
        content = {
            "param": {
                "subspace": "baseapp",
                "key": "BlockParams",
                "value": '{"max_bytes":"200000","max_gas":"2000000"}',
            },
        }
        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryParamsResponse())

        params = ParamsRestClient(mock_client)

        assert (
            params.Params(QueryParamsRequest(subspace="baseapp", key="BlockParams"))
            == expected_response
        )
        assert mock_client.last_base_url == "/cosmos/params/v1beta1/params"



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
import argparse
from time import sleep

import requests

from cosmpy.aerial.client import LedgerClient, NetworkConfig
from cosmpy.aerial.contract import LedgerContract
from cosmpy.aerial.wallet import LocalWallet
from cosmpy.crypto.address import Address
from cosmpy.crypto.keypairs import PrivateKey

COIN_PRICE_URL = (
    "https://api.coingecko.com/api/v3/simple/price?ids=fetch-ai&vs_currencies=usd"
)
UPDATE_INTERVAL_SECONDS = 10
ORACLE_VALUE_DECIMALS = 5


def _parse_commandline():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "contract_path", help="The path to the oracle contract to upload"
    )
    parser.add_argument(
        "contract_address",
        nargs="?",
        type=Address,
        help="The address of the oracle contract if already deployed",
    )
    return parser.parse_args()


def main():
    args = _parse_commandline()

    wallet = LocalWallet(PrivateKey("T7w1yHq1QIcQiSqV27YSwk+i1i+Y4JMKhkpawCQIh6s="))

    ledger = LedgerClient(NetworkConfig.fetchai_stable_testnet())

    contract = LedgerContract(args.contract_path, ledger, address=args.contract_address)

    if not args.contract_address:
        instantiation_message = {"fee": "100"}
        contract.deploy(instantiation_message, wallet, funds="1atestfet")

    print(f"Oracle contract deployed at: {contract.address}")

    grant_role_message = {"grant_oracle_role": {"address": str(wallet.address())}}
    contract.execute(grant_role_message, wallet).wait_to_complete()

    print(f"Oracle role granted to address: {str(wallet.address())}")

    while True:
        resp = requests.get(COIN_PRICE_URL).json()
        price = resp["fetch-ai"]["usd"]
        value = int(price * 10**ORACLE_VALUE_DECIMALS)

        update_message = {
            "update_oracle_value": {
                "value": str(value),
                "decimals": str(ORACLE_VALUE_DECIMALS),
            }
        }
        contract.execute(update_message, wallet).wait_to_complete()

        print(f"Oracle value updated to: {price} USD")
        print(f"Next update in {UPDATE_INTERVAL_SECONDS} seconds...")
        sleep(UPDATE_INTERVAL_SECONDS)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#   Modifications copyright (C) 2022 Cros-Nest
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Implementation of Mint interface using REST."""
import base64
import json
from typing import Union

from google.protobuf.json_format import Parse

from cosmpy.common.rest_client import RestClient
from cosmpy.mint.interface import Mint
from cosmpy.protos.cosmos.mint.v1beta1.query_pb2 import (
    QueryAnnualProvisionsResponse,
    QueryInflationResponse,
    QueryParamsResponse,
)


def isNumber(value: Union[str, bytes]) -> bool:
    """
    Check is string ob bytes is number.

    :param value: str, bytes
    :return: bool
    """
    try:
        float(str(value))
        return True
    except ValueError:
        return False


class MintRestClient(Mint):
    """Mint REST client."""

    API_URL = "/cosmos/mint/v1beta1"

    def __init__(self, rest_api: RestClient) -> None:
        """
        Initialize.

        :param rest_api: RestClient api
        """
        self._rest_api = rest_api

    def AnnualProvisions(self) -> QueryAnnualProvisionsResponse:
        """
        AnnualProvisions current minting annual provisions value.
        :return: a QueryAnnualProvisionsResponse instance
        """
        json_response = self._rest_api.get(f"{self.API_URL}/annual_provisions")
        # The QueryAnnualProvisionsResponse expect a base64 encoded value
        # but the Rest endpoint return digits
        j = json.loads(json_response)
        if isNumber(j["annual_provisions"]):
            j["annual_provisions"] = base64.b64encode(
                j["annual_provisions"].encode()
            ).decode("utf8")
        json_response = json.dumps(j).encode("utf-8")

        return Parse(json_response, QueryAnnualProvisionsResponse())

    def Inflation(self) -> QueryInflationResponse:
        """
        Inflation returns the current minting inflation value.
        :return: a QueryInflationResponse instance
        """
        json_response = self._rest_api.get(f"{self.API_URL}/inflation")
        # The QueryInflationResponse expect a base64 encoded value
        # but the Rest endpoint return digits
        j = json.loads(json_response)
        if isNumber(j["inflation"]):
            j["inflation"] = base64.b64encode(j["inflation"].encode()).decode("utf8")
        json_response = json.dumps(j).encode("utf-8")

        return Parse(json_response, QueryInflationResponse())

    def Params(self) -> QueryParamsResponse:
        """
        Params queries params of the Mint module.
        :return: a QueryParamsResponse instance
        """
        json_response = self._rest_api.get(f"{self.API_URL}/params")
        return Parse(json_response, QueryParamsResponse())



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cosmos/base/snapshots/v1beta1/snapshot.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='cosmos/base/snapshots/v1beta1/snapshot.proto',
  package='cosmos.base.snapshots.v1beta1',
  syntax='proto3',
  serialized_options=b'Z,github.com/cosmos/cosmos-sdk/snapshots/types',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n,cosmos/base/snapshots/v1beta1/snapshot.proto\x12\x1d\x63osmos.base.snapshots.v1beta1\x1a\x14gogoproto/gogo.proto\"\x89\x01\n\x08Snapshot\x12\x0e\n\x06height\x18\x01 \x01(\x04\x12\x0e\n\x06\x66ormat\x18\x02 \x01(\r\x12\x0e\n\x06\x63hunks\x18\x03 \x01(\r\x12\x0c\n\x04hash\x18\x04 \x01(\x0c\x12?\n\x08metadata\x18\x05 \x01(\x0b\x32\'.cosmos.base.snapshots.v1beta1.MetadataB\x04\xc8\xde\x1f\x00\" \n\x08Metadata\x12\x14\n\x0c\x63hunk_hashes\x18\x01 \x03(\x0c\x42.Z,github.com/cosmos/cosmos-sdk/snapshots/typesb\x06proto3'
  ,
  dependencies=[gogoproto_dot_gogo__pb2.DESCRIPTOR,])




_SNAPSHOT = _descriptor.Descriptor(
  name='Snapshot',
  full_name='cosmos.base.snapshots.v1beta1.Snapshot',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='height', full_name='cosmos.base.snapshots.v1beta1.Snapshot.height', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='format', full_name='cosmos.base.snapshots.v1beta1.Snapshot.format', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='chunks', full_name='cosmos.base.snapshots.v1beta1.Snapshot.chunks', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hash', full_name='cosmos.base.snapshots.v1beta1.Snapshot.hash', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='cosmos.base.snapshots.v1beta1.Snapshot.metadata', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=102,
  serialized_end=239,
)


_METADATA = _descriptor.Descriptor(
  name='Metadata',
  full_name='cosmos.base.snapshots.v1beta1.Metadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='chunk_hashes', full_name='cosmos.base.snapshots.v1beta1.Metadata.chunk_hashes', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=241,
  serialized_end=273,
)

_SNAPSHOT.fields_by_name['metadata'].message_type = _METADATA
DESCRIPTOR.message_types_by_name['Snapshot'] = _SNAPSHOT
DESCRIPTOR.message_types_by_name['Metadata'] = _METADATA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Snapshot = _reflection.GeneratedProtocolMessageType('Snapshot', (_message.Message,), {
  'DESCRIPTOR' : _SNAPSHOT,
  '__module__' : 'cosmos.base.snapshots.v1beta1.snapshot_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.base.snapshots.v1beta1.Snapshot)
  })
_sym_db.RegisterMessage(Snapshot)

Metadata = _reflection.GeneratedProtocolMessageType('Metadata', (_message.Message,), {
  'DESCRIPTOR' : _METADATA,
  '__module__' : 'cosmos.base.snapshots.v1beta1.snapshot_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.base.snapshots.v1beta1.Metadata)
  })
_sym_db.RegisterMessage(Metadata)


DESCRIPTOR._options = None
_SNAPSHOT.fields_by_name['metadata']._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cosmwasm/wasm/v1/types.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2
from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='cosmwasm/wasm/v1/types.proto',
  package='cosmwasm.wasm.v1',
  syntax='proto3',
  serialized_options=b'Z&github.com/CosmWasm/wasmd/x/wasm/types\310\341\036\000\250\342\036\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1c\x63osmwasm/wasm/v1/types.proto\x12\x10\x63osmwasm.wasm.v1\x1a\x19\x63osmos_proto/cosmos.proto\x1a\x14gogoproto/gogo.proto\x1a\x19google/protobuf/any.proto\"V\n\x0f\x41\x63\x63\x65ssTypeParam\x12=\n\x05value\x18\x01 \x01(\x0e\x32\x1c.cosmwasm.wasm.v1.AccessTypeB\x10\xf2\xde\x1f\x0cyaml:\"value\":\x04\x98\xa0\x1f\x01\"\x82\x01\n\x0c\x41\x63\x63\x65ssConfig\x12G\n\npermission\x18\x01 \x01(\x0e\x32\x1c.cosmwasm.wasm.v1.AccessTypeB\x15\xf2\xde\x1f\x11yaml:\"permission\"\x12#\n\x07\x61\x64\x64ress\x18\x02 \x01(\tB\x12\xf2\xde\x1f\x0eyaml:\"address\":\x04\x98\xa0\x1f\x01\"\x99\x02\n\x06Params\x12]\n\x12\x63ode_upload_access\x18\x01 \x01(\x0b\x32\x1e.cosmwasm.wasm.v1.AccessConfigB!\xc8\xde\x1f\x00\xf2\xde\x1f\x19yaml:\"code_upload_access\"\x12o\n\x1einstantiate_default_permission\x18\x02 \x01(\x0e\x32\x1c.cosmwasm.wasm.v1.AccessTypeB)\xf2\xde\x1f%yaml:\"instantiate_default_permission\"\x12\x39\n\x12max_wasm_code_size\x18\x03 \x01(\x04\x42\x1d\xf2\xde\x1f\x19yaml:\"max_wasm_code_size\":\x04\x98\xa0\x1f\x00\"|\n\x08\x43odeInfo\x12\x11\n\tcode_hash\x18\x01 \x01(\x0c\x12\x0f\n\x07\x63reator\x18\x02 \x01(\t\x12@\n\x12instantiate_config\x18\x05 \x01(\x0b\x32\x1e.cosmwasm.wasm.v1.AccessConfigB\x04\xc8\xde\x1f\x00J\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05\"\xff\x01\n\x0c\x43ontractInfo\x12\x1b\n\x07\x63ode_id\x18\x01 \x01(\x04\x42\n\xe2\xde\x1f\x06\x43odeID\x12\x0f\n\x07\x63reator\x18\x02 \x01(\t\x12\r\n\x05\x61\x64min\x18\x03 \x01(\t\x12\r\n\x05label\x18\x04 \x01(\t\x12\x35\n\x07\x63reated\x18\x05 \x01(\x0b\x32$.cosmwasm.wasm.v1.AbsoluteTxPosition\x12\"\n\x0bibc_port_id\x18\x06 \x01(\tB\r\xe2\xde\x1f\tIBCPortID\x12\x42\n\textension\x18\x07 \x01(\x0b\x32\x14.google.protobuf.AnyB\x19\xca\xb4-\x15\x43ontractInfoExtension:\x04\xe8\xa0\x1f\x01\"\xda\x01\n\x18\x43ontractCodeHistoryEntry\x12\x45\n\toperation\x18\x01 \x01(\x0e\x32\x32.cosmwasm.wasm.v1.ContractCodeHistoryOperationType\x12\x1b\n\x07\x63ode_id\x18\x02 \x01(\x04\x42\n\xe2\xde\x1f\x06\x43odeID\x12\x35\n\x07updated\x18\x03 \x01(\x0b\x32$.cosmwasm.wasm.v1.AbsoluteTxPosition\x12#\n\x03msg\x18\x04 \x01(\x0c\x42\x16\xfa\xde\x1f\x12RawContractMessage\"<\n\x12\x41\x62soluteTxPosition\x12\x14\n\x0c\x62lock_height\x18\x01 \x01(\x04\x12\x10\n\x08tx_index\x18\x02 \x01(\x04\"]\n\x05Model\x12\x45\n\x03key\x18\x01 \x01(\x0c\x42\x38\xfa\xde\x1f\x34github.com/tendermint/tendermint/libs/bytes.HexBytes\x12\r\n\x05value\x18\x02 \x01(\x0c*\xe9\x01\n\nAccessType\x12\x36\n\x17\x41\x43\x43\x45SS_TYPE_UNSPECIFIED\x10\x00\x1a\x19\x8a\x9d \x15\x41\x63\x63\x65ssTypeUnspecified\x12,\n\x12\x41\x43\x43\x45SS_TYPE_NOBODY\x10\x01\x1a\x14\x8a\x9d \x10\x41\x63\x63\x65ssTypeNobody\x12\x37\n\x18\x41\x43\x43\x45SS_TYPE_ONLY_ADDRESS\x10\x02\x1a\x19\x8a\x9d \x15\x41\x63\x63\x65ssTypeOnlyAddress\x12\x32\n\x15\x41\x43\x43\x45SS_TYPE_EVERYBODY\x10\x03\x1a\x17\x8a\x9d \x13\x41\x63\x63\x65ssTypeEverybody\x1a\x08\x88\xa3\x1e\x00\xa8\xa4\x1e\x00*\xa6\x03\n ContractCodeHistoryOperationType\x12\x65\n0CONTRACT_CODE_HISTORY_OPERATION_TYPE_UNSPECIFIED\x10\x00\x1a/\x8a\x9d +ContractCodeHistoryOperationTypeUnspecified\x12W\n)CONTRACT_CODE_HISTORY_OPERATION_TYPE_INIT\x10\x01\x1a(\x8a\x9d $ContractCodeHistoryOperationTypeInit\x12]\n,CONTRACT_CODE_HISTORY_OPERATION_TYPE_MIGRATE\x10\x02\x1a+\x8a\x9d \'ContractCodeHistoryOperationTypeMigrate\x12]\n,CONTRACT_CODE_HISTORY_OPERATION_TYPE_GENESIS\x10\x03\x1a+\x8a\x9d \'ContractCodeHistoryOperationTypeGenesis\x1a\x04\x88\xa3\x1e\x00\x42\x30Z&github.com/CosmWasm/wasmd/x/wasm/types\xc8\xe1\x1e\x00\xa8\xe2\x1e\x01\x62\x06proto3'
  ,
  dependencies=[cosmos__proto_dot_cosmos__pb2.DESCRIPTOR,gogoproto_dot_gogo__pb2.DESCRIPTOR,google_dot_protobuf_dot_any__pb2.DESCRIPTOR,])

_ACCESSTYPE = _descriptor.EnumDescriptor(
  name='AccessType',
  full_name='cosmwasm.wasm.v1.AccessType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ACCESS_TYPE_UNSPECIFIED', index=0, number=0,
      serialized_options=b'\212\235 \025AccessTypeUnspecified',
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ACCESS_TYPE_NOBODY', index=1, number=1,
      serialized_options=b'\212\235 \020AccessTypeNobody',
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ACCESS_TYPE_ONLY_ADDRESS', index=2, number=2,
      serialized_options=b'\212\235 \025AccessTypeOnlyAddress',
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ACCESS_TYPE_EVERYBODY', index=3, number=3,
      serialized_options=b'\212\235 \023AccessTypeEverybody',
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=b'\210\243\036\000\250\244\036\000',
  serialized_start=1394,
  serialized_end=1627,
)
_sym_db.RegisterEnumDescriptor(_ACCESSTYPE)

AccessType = enum_type_wrapper.EnumTypeWrapper(_ACCESSTYPE)
_CONTRACTCODEHISTORYOPERATIONTYPE = _descriptor.EnumDescriptor(
  name='ContractCodeHistoryOperationType',
  full_name='cosmwasm.wasm.v1.ContractCodeHistoryOperationType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='CONTRACT_CODE_HISTORY_OPERATION_TYPE_UNSPECIFIED', index=0, number=0,
      serialized_options=b'\212\235 +ContractCodeHistoryOperationTypeUnspecified',
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CONTRACT_CODE_HISTORY_OPERATION_TYPE_INIT', index=1, number=1,
      serialized_options=b'\212\235 $ContractCodeHistoryOperationTypeInit',
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CONTRACT_CODE_HISTORY_OPERATION_TYPE_MIGRATE', index=2, number=2,
      serialized_options=b'\212\235 \'ContractCodeHistoryOperationTypeMigrate',
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CONTRACT_CODE_HISTORY_OPERATION_TYPE_GENESIS', index=3, number=3,
      serialized_options=b'\212\235 \'ContractCodeHistoryOperationTypeGenesis',
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=b'\210\243\036\000',
  serialized_start=1630,
  serialized_end=2052,
)
_sym_db.RegisterEnumDescriptor(_CONTRACTCODEHISTORYOPERATIONTYPE)

ContractCodeHistoryOperationType = enum_type_wrapper.EnumTypeWrapper(_CONTRACTCODEHISTORYOPERATIONTYPE)
ACCESS_TYPE_UNSPECIFIED = 0
ACCESS_TYPE_NOBODY = 1
ACCESS_TYPE_ONLY_ADDRESS = 2
ACCESS_TYPE_EVERYBODY = 3
CONTRACT_CODE_HISTORY_OPERATION_TYPE_UNSPECIFIED = 0
CONTRACT_CODE_HISTORY_OPERATION_TYPE_INIT = 1
CONTRACT_CODE_HISTORY_OPERATION_TYPE_MIGRATE = 2
CONTRACT_CODE_HISTORY_OPERATION_TYPE_GENESIS = 3



_ACCESSTYPEPARAM = _descriptor.Descriptor(
  name='AccessTypeParam',
  full_name='cosmwasm.wasm.v1.AccessTypeParam',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='cosmwasm.wasm.v1.AccessTypeParam.value', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\014yaml:\"value\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\230\240\037\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=126,
  serialized_end=212,
)


_ACCESSCONFIG = _descriptor.Descriptor(
  name='AccessConfig',
  full_name='cosmwasm.wasm.v1.AccessConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='permission', full_name='cosmwasm.wasm.v1.AccessConfig.permission', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\021yaml:\"permission\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='address', full_name='cosmwasm.wasm.v1.AccessConfig.address', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\016yaml:\"address\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\230\240\037\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=215,
  serialized_end=345,
)


_PARAMS = _descriptor.Descriptor(
  name='Params',
  full_name='cosmwasm.wasm.v1.Params',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='code_upload_access', full_name='cosmwasm.wasm.v1.Params.code_upload_access', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000\362\336\037\031yaml:\"code_upload_access\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='instantiate_default_permission', full_name='cosmwasm.wasm.v1.Params.instantiate_default_permission', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037%yaml:\"instantiate_default_permission\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max_wasm_code_size', full_name='cosmwasm.wasm.v1.Params.max_wasm_code_size', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\031yaml:\"max_wasm_code_size\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\230\240\037\000',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=348,
  serialized_end=629,
)


_CODEINFO = _descriptor.Descriptor(
  name='CodeInfo',
  full_name='cosmwasm.wasm.v1.CodeInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='code_hash', full_name='cosmwasm.wasm.v1.CodeInfo.code_hash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='creator', full_name='cosmwasm.wasm.v1.CodeInfo.creator', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='instantiate_config', full_name='cosmwasm.wasm.v1.CodeInfo.instantiate_config', index=2,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=631,
  serialized_end=755,
)


_CONTRACTINFO = _descriptor.Descriptor(
  name='ContractInfo',
  full_name='cosmwasm.wasm.v1.ContractInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='code_id', full_name='cosmwasm.wasm.v1.ContractInfo.code_id', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\342\336\037\006CodeID', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='creator', full_name='cosmwasm.wasm.v1.ContractInfo.creator', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='admin', full_name='cosmwasm.wasm.v1.ContractInfo.admin', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='label', full_name='cosmwasm.wasm.v1.ContractInfo.label', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='created', full_name='cosmwasm.wasm.v1.ContractInfo.created', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ibc_port_id', full_name='cosmwasm.wasm.v1.ContractInfo.ibc_port_id', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\342\336\037\tIBCPortID', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='extension', full_name='cosmwasm.wasm.v1.ContractInfo.extension', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\312\264-\025ContractInfoExtension', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\350\240\037\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=758,
  serialized_end=1013,
)


_CONTRACTCODEHISTORYENTRY = _descriptor.Descriptor(
  name='ContractCodeHistoryEntry',
  full_name='cosmwasm.wasm.v1.ContractCodeHistoryEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='operation', full_name='cosmwasm.wasm.v1.ContractCodeHistoryEntry.operation', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='code_id', full_name='cosmwasm.wasm.v1.ContractCodeHistoryEntry.code_id', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\342\336\037\006CodeID', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='updated', full_name='cosmwasm.wasm.v1.ContractCodeHistoryEntry.updated', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='msg', full_name='cosmwasm.wasm.v1.ContractCodeHistoryEntry.msg', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372\336\037\022RawContractMessage', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1016,
  serialized_end=1234,
)


_ABSOLUTETXPOSITION = _descriptor.Descriptor(
  name='AbsoluteTxPosition',
  full_name='cosmwasm.wasm.v1.AbsoluteTxPosition',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='block_height', full_name='cosmwasm.wasm.v1.AbsoluteTxPosition.block_height', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tx_index', full_name='cosmwasm.wasm.v1.AbsoluteTxPosition.tx_index', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1236,
  serialized_end=1296,
)


_MODEL = _descriptor.Descriptor(
  name='Model',
  full_name='cosmwasm.wasm.v1.Model',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='cosmwasm.wasm.v1.Model.key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372\336\0374github.com/tendermint/tendermint/libs/bytes.HexBytes', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='cosmwasm.wasm.v1.Model.value', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1298,
  serialized_end=1391,
)

_ACCESSTYPEPARAM.fields_by_name['value'].enum_type = _ACCESSTYPE
_ACCESSCONFIG.fields_by_name['permission'].enum_type = _ACCESSTYPE
_PARAMS.fields_by_name['code_upload_access'].message_type = _ACCESSCONFIG
_PARAMS.fields_by_name['instantiate_default_permission'].enum_type = _ACCESSTYPE
_CODEINFO.fields_by_name['instantiate_config'].message_type = _ACCESSCONFIG
_CONTRACTINFO.fields_by_name['created'].message_type = _ABSOLUTETXPOSITION
_CONTRACTINFO.fields_by_name['extension'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_CONTRACTCODEHISTORYENTRY.fields_by_name['operation'].enum_type = _CONTRACTCODEHISTORYOPERATIONTYPE
_CONTRACTCODEHISTORYENTRY.fields_by_name['updated'].message_type = _ABSOLUTETXPOSITION
DESCRIPTOR.message_types_by_name['AccessTypeParam'] = _ACCESSTYPEPARAM
DESCRIPTOR.message_types_by_name['AccessConfig'] = _ACCESSCONFIG
DESCRIPTOR.message_types_by_name['Params'] = _PARAMS
DESCRIPTOR.message_types_by_name['CodeInfo'] = _CODEINFO
DESCRIPTOR.message_types_by_name['ContractInfo'] = _CONTRACTINFO
DESCRIPTOR.message_types_by_name['ContractCodeHistoryEntry'] = _CONTRACTCODEHISTORYENTRY
DESCRIPTOR.message_types_by_name['AbsoluteTxPosition'] = _ABSOLUTETXPOSITION
DESCRIPTOR.message_types_by_name['Model'] = _MODEL
DESCRIPTOR.enum_types_by_name['AccessType'] = _ACCESSTYPE
DESCRIPTOR.enum_types_by_name['ContractCodeHistoryOperationType'] = _CONTRACTCODEHISTORYOPERATIONTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

AccessTypeParam = _reflection.GeneratedProtocolMessageType('AccessTypeParam', (_message.Message,), {
  'DESCRIPTOR' : _ACCESSTYPEPARAM,
  '__module__' : 'cosmwasm.wasm.v1.types_pb2'
  # @@protoc_insertion_point(class_scope:cosmwasm.wasm.v1.AccessTypeParam)
  })
_sym_db.RegisterMessage(AccessTypeParam)

AccessConfig = _reflection.GeneratedProtocolMessageType('AccessConfig', (_message.Message,), {
  'DESCRIPTOR' : _ACCESSCONFIG,
  '__module__' : 'cosmwasm.wasm.v1.types_pb2'
  # @@protoc_insertion_point(class_scope:cosmwasm.wasm.v1.AccessConfig)
  })
_sym_db.RegisterMessage(AccessConfig)

Params = _reflection.GeneratedProtocolMessageType('Params', (_message.Message,), {
  'DESCRIPTOR' : _PARAMS,
  '__module__' : 'cosmwasm.wasm.v1.types_pb2'
  # @@protoc_insertion_point(class_scope:cosmwasm.wasm.v1.Params)
  })
_sym_db.RegisterMessage(Params)

CodeInfo = _reflection.GeneratedProtocolMessageType('CodeInfo', (_message.Message,), {
  'DESCRIPTOR' : _CODEINFO,
  '__module__' : 'cosmwasm.wasm.v1.types_pb2'
  # @@protoc_insertion_point(class_scope:cosmwasm.wasm.v1.CodeInfo)
  })
_sym_db.RegisterMessage(CodeInfo)

ContractInfo = _reflection.GeneratedProtocolMessageType('ContractInfo', (_message.Message,), {
  'DESCRIPTOR' : _CONTRACTINFO,
  '__module__' : 'cosmwasm.wasm.v1.types_pb2'
  # @@protoc_insertion_point(class_scope:cosmwasm.wasm.v1.ContractInfo)
  })
_sym_db.RegisterMessage(ContractInfo)

ContractCodeHistoryEntry = _reflection.GeneratedProtocolMessageType('ContractCodeHistoryEntry', (_message.Message,), {
  'DESCRIPTOR' : _CONTRACTCODEHISTORYENTRY,
  '__module__' : 'cosmwasm.wasm.v1.types_pb2'
  # @@protoc_insertion_point(class_scope:cosmwasm.wasm.v1.ContractCodeHistoryEntry)
  })
_sym_db.RegisterMessage(ContractCodeHistoryEntry)

AbsoluteTxPosition = _reflection.GeneratedProtocolMessageType('AbsoluteTxPosition', (_message.Message,), {
  'DESCRIPTOR' : _ABSOLUTETXPOSITION,
  '__module__' : 'cosmwasm.wasm.v1.types_pb2'
  # @@protoc_insertion_point(class_scope:cosmwasm.wasm.v1.AbsoluteTxPosition)
  })
_sym_db.RegisterMessage(AbsoluteTxPosition)

Model = _reflection.GeneratedProtocolMessageType('Model', (_message.Message,), {
  'DESCRIPTOR' : _MODEL,
  '__module__' : 'cosmwasm.wasm.v1.types_pb2'
  # @@protoc_insertion_point(class_scope:cosmwasm.wasm.v1.Model)
  })
_sym_db.RegisterMessage(Model)


DESCRIPTOR._options = None
_ACCESSTYPE._options = None
_ACCESSTYPE.values_by_name["ACCESS_TYPE_UNSPECIFIED"]._options = None
_ACCESSTYPE.values_by_name["ACCESS_TYPE_NOBODY"]._options = None
_ACCESSTYPE.values_by_name["ACCESS_TYPE_ONLY_ADDRESS"]._options = None
_ACCESSTYPE.values_by_name["ACCESS_TYPE_EVERYBODY"]._options = None
_CONTRACTCODEHISTORYOPERATIONTYPE._options = None
_CONTRACTCODEHISTORYOPERATIONTYPE.values_by_name["CONTRACT_CODE_HISTORY_OPERATION_TYPE_UNSPECIFIED"]._options = None
_CONTRACTCODEHISTORYOPERATIONTYPE.values_by_name["CONTRACT_CODE_HISTORY_OPERATION_TYPE_INIT"]._options = None
_CONTRACTCODEHISTORYOPERATIONTYPE.values_by_name["CONTRACT_CODE_HISTORY_OPERATION_TYPE_MIGRATE"]._options = None
_CONTRACTCODEHISTORYOPERATIONTYPE.values_by_name["CONTRACT_CODE_HISTORY_OPERATION_TYPE_GENESIS"]._options = None
_ACCESSTYPEPARAM.fields_by_name['value']._options = None
_ACCESSTYPEPARAM._options = None
_ACCESSCONFIG.fields_by_name['permission']._options = None
_ACCESSCONFIG.fields_by_name['address']._options = None
_ACCESSCONFIG._options = None
_PARAMS.fields_by_name['code_upload_access']._options = None
_PARAMS.fields_by_name['instantiate_default_permission']._options = None
_PARAMS.fields_by_name['max_wasm_code_size']._options = None
_PARAMS._options = None
_CODEINFO.fields_by_name['instantiate_config']._options = None
_CONTRACTINFO.fields_by_name['code_id']._options = None
_CONTRACTINFO.fields_by_name['ibc_port_id']._options = None
_CONTRACTINFO.fields_by_name['extension']._options = None
_CONTRACTINFO._options = None
_CONTRACTCODEHISTORYENTRY.fields_by_name['code_id']._options = None
_CONTRACTCODEHISTORYENTRY.fields_by_name['msg']._options = None
_MODEL.fields_by_name['key']._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

from abc import ABC, abstractmethod
from typing import Optional

from bip_utils import Bip39SeedGenerator, Bip44, Bip44Coins  # type: ignore

from cosmpy.crypto.address import Address
from cosmpy.crypto.hashfuncs import sha256
from cosmpy.crypto.interface import Signer
from cosmpy.crypto.keypairs import PrivateKey, PublicKey


class Wallet(ABC):
    @abstractmethod
    def address(self) -> Address:
        pass

    @abstractmethod
    def public_key(self) -> PublicKey:
        pass

    @abstractmethod
    def signer(self) -> Signer:
        pass


class LocalWallet(Wallet):
    @staticmethod
    def generate(prefix: Optional[str] = None) -> "LocalWallet":
        return LocalWallet(PrivateKey(), prefix=prefix)

    @staticmethod
    def from_mnemonic(mnemonic: str, prefix: Optional[str] = None) -> "LocalWallet":
        seed_bytes = Bip39SeedGenerator(mnemonic).Generate()
        bip44_def_ctx = Bip44.FromSeed(
            seed_bytes, Bip44Coins.COSMOS
        ).DeriveDefaultPath()
        return LocalWallet(
            PrivateKey(bip44_def_ctx.PrivateKey().Raw().ToBytes()), prefix=prefix
        )

    @staticmethod
    def from_unsafe_seed(
        text: str, index: Optional[int] = None, prefix: Optional[str] = None
    ) -> "LocalWallet":
        private_key_bytes = sha256(text.encode())
        if index is not None:
            private_key_bytes = sha256(
                private_key_bytes + index.to_bytes(4, byteorder="big")
            )
        return LocalWallet(PrivateKey(private_key_bytes), prefix=prefix)

    def __init__(self, private_key: PrivateKey, prefix: Optional[str] = None):
        self._private_key = private_key
        self._prefix = prefix

    def address(self) -> Address:
        return Address(self._private_key, self._prefix)

    def public_key(self) -> PublicKey:
        return self._private_key

    def signer(self) -> PrivateKey:
        return self._private_key



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2022 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Interface for the Slashing functionality of CosmosSDK."""

from abc import ABC, abstractmethod

from cosmpy.protos.cosmos.slashing.v1beta1.query_pb2 import (
    QueryParamsResponse,
    QuerySigningInfoRequest,
    QuerySigningInfoResponse,
    QuerySigningInfosRequest,
    QuerySigningInfosResponse,
)


class Slashing(ABC):
    """Slashing abstract class."""

    @abstractmethod
    def Params(self) -> QueryParamsResponse:
        """
        Params queries the parameters of slashing module

        :return: QueryParamsResponse
        """

    @abstractmethod
    def SigningInfo(self, request: QuerySigningInfoRequest) -> QuerySigningInfoResponse:
        """
        SigningInfo queries the signing info of given cons address.

        :param request: QuerySigningInfoRequest

        :return: QuerySigningInfoResponse
        """

    @abstractmethod
    def SigningInfos(
        self, request: QuerySigningInfosRequest
    ) -> QuerySigningInfosResponse:
        """
        SigningInfos queries signing info of all validators.

        :param request: QuerySigningInfosRequest

        :return: QuerySigningInfosResponse
        """



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#   Modifications copyright (C) 2022 Cros-Nest
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Tests for REST implementation of Mint."""
import base64
import json
import unittest

from google.protobuf.json_format import ParseDict

from cosmpy.mint.rest_client import MintRestClient
from cosmpy.protos.cosmos.mint.v1beta1.query_pb2 import (
    QueryAnnualProvisionsResponse,
    QueryInflationResponse,
    QueryParamsResponse,
)
from tests.helpers import MockRestClient


class MintRestClientTestCase(unittest.TestCase):
    """Test case of Mint module."""

    @staticmethod
    def test_AnnualProvisionsBase64():
        """Test query annual provision for the positive result."""
        content = {
            "annual_provisions": "MTIzNA=="
        }  # use value "1234" in base64 encoded format

        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryAnnualProvisionsResponse())

        mint = MintRestClient(mock_client)

        assert mint.AnnualProvisions() == expected_response
        assert expected_response.annual_provisions == b"1234"
        assert mock_client.last_base_url == "/cosmos/mint/v1beta1/annual_provisions"

    @staticmethod
    def test_AnnualProvisionsInteger():
        """Test query annual provision for the positive result."""
        content = {"annual_provisions": "1234"}

        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryAnnualProvisionsResponse())
        # The AnnualProvisions object is expecting a base64 encoded value
        expected_response.annual_provisions = base64.b64encode(
            expected_response.annual_provisions
        )

        mint = MintRestClient(mock_client)
        provision = mint.AnnualProvisions()
        assert provision == expected_response
        assert expected_response.annual_provisions == b"1234"
        assert mock_client.last_base_url == "/cosmos/mint/v1beta1/annual_provisions"

    @staticmethod
    def test_InflationBase64():
        """Test query inflation for the positive result."""
        content = {"inflation": "MC4wMTIzNDU="}

        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryInflationResponse())

        mint = MintRestClient(mock_client)

        assert mint.Inflation() == expected_response
        assert mint.Inflation().inflation == b"0.012345"
        assert mock_client.last_base_url == "/cosmos/mint/v1beta1/inflation"

    @staticmethod
    def test_InflationInteger():
        """Test query inflation for the positive result."""
        content = {"inflation": "0.012345"}

        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryInflationResponse())
        # The Inflation object is expecting a base64 encoded value
        # FIXME: The create an issue by loosing the decimal dot and adding padding '='  # pylint: disable=W0511
        expected_response.inflation = base64.b64encode(expected_response.inflation)

        mint = MintRestClient(mock_client)

        # This test is expected to fail because of the loss of the decimal dot.
        # assert mint.Inflation() == expected_response
        assert mint.Inflation().inflation == b"0.012345"
        assert mock_client.last_base_url == "/cosmos/mint/v1beta1/inflation"

    @staticmethod
    def test_query_params():
        """Test query params for the positive result."""
        content = {
            "params": {
                "mintDenom": "string",
                "inflationRate": "0.12345",
                "blocksPerYear": "1234",
            }
        }
        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryParamsResponse())

        mint = MintRestClient(mock_client)

        assert mint.Params().params.blocks_per_year == 1234
        assert mint.Params().params.inflation_rate == "0.12345"
        assert mint.Params().params.mint_denom == "string"
        assert mint.Params() == expected_response
        assert mock_client.last_base_url == "/cosmos/mint/v1beta1/params"



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ibc/applications/transfer/v1/query.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from cosmos.base.query.v1beta1 import pagination_pb2 as cosmos_dot_base_dot_query_dot_v1beta1_dot_pagination__pb2
from ibc.applications.transfer.v1 import transfer_pb2 as ibc_dot_applications_dot_transfer_dot_v1_dot_transfer__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='ibc/applications/transfer/v1/query.proto',
  package='ibc.applications.transfer.v1',
  syntax='proto3',
  serialized_options=b'Z7github.com/cosmos/ibc-go/v2/modules/apps/transfer/types',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n(ibc/applications/transfer/v1/query.proto\x12\x1cibc.applications.transfer.v1\x1a\x14gogoproto/gogo.proto\x1a*cosmos/base/query/v1beta1/pagination.proto\x1a+ibc/applications/transfer/v1/transfer.proto\x1a\x1cgoogle/api/annotations.proto\"&\n\x16QueryDenomTraceRequest\x12\x0c\n\x04hash\x18\x01 \x01(\t\"X\n\x17QueryDenomTraceResponse\x12=\n\x0b\x64\x65nom_trace\x18\x01 \x01(\x0b\x32(.ibc.applications.transfer.v1.DenomTrace\"U\n\x17QueryDenomTracesRequest\x12:\n\npagination\x18\x01 \x01(\x0b\x32&.cosmos.base.query.v1beta1.PageRequest\"\xa7\x01\n\x18QueryDenomTracesResponse\x12N\n\x0c\x64\x65nom_traces\x18\x01 \x03(\x0b\x32(.ibc.applications.transfer.v1.DenomTraceB\x0e\xaa\xdf\x1f\x06Traces\xc8\xde\x1f\x00\x12;\n\npagination\x18\x02 \x01(\x0b\x32\'.cosmos.base.query.v1beta1.PageResponse\"\x14\n\x12QueryParamsRequest\"K\n\x13QueryParamsResponse\x12\x34\n\x06params\x18\x01 \x01(\x0b\x32$.ibc.applications.transfer.v1.Params\"&\n\x15QueryDenomHashRequest\x12\r\n\x05trace\x18\x01 \x01(\t\"&\n\x16QueryDenomHashResponse\x12\x0c\n\x04hash\x18\x01 \x01(\t2\xa4\x05\n\x05Query\x12\xac\x01\n\nDenomTrace\x12\x34.ibc.applications.transfer.v1.QueryDenomTraceRequest\x1a\x35.ibc.applications.transfer.v1.QueryDenomTraceResponse\"1\x82\xd3\xe4\x93\x02+\x12)/ibc/apps/transfer/v1/denom_traces/{hash}\x12\xa8\x01\n\x0b\x44\x65nomTraces\x12\x35.ibc.applications.transfer.v1.QueryDenomTracesRequest\x1a\x36.ibc.applications.transfer.v1.QueryDenomTracesResponse\"*\x82\xd3\xe4\x93\x02$\x12\"/ibc/apps/transfer/v1/denom_traces\x12\x93\x01\n\x06Params\x12\x30.ibc.applications.transfer.v1.QueryParamsRequest\x1a\x31.ibc.applications.transfer.v1.QueryParamsResponse\"$\x82\xd3\xe4\x93\x02\x1e\x12\x1c/ibc/apps/transfer/v1/params\x12\xaa\x01\n\tDenomHash\x12\x33.ibc.applications.transfer.v1.QueryDenomHashRequest\x1a\x34.ibc.applications.transfer.v1.QueryDenomHashResponse\"2\x82\xd3\xe4\x93\x02,\x12*/ibc/apps/transfer/v1/denom_hashes/{trace}B9Z7github.com/cosmos/ibc-go/v2/modules/apps/transfer/typesb\x06proto3'
  ,
  dependencies=[gogoproto_dot_gogo__pb2.DESCRIPTOR,cosmos_dot_base_dot_query_dot_v1beta1_dot_pagination__pb2.DESCRIPTOR,ibc_dot_applications_dot_transfer_dot_v1_dot_transfer__pb2.DESCRIPTOR,google_dot_api_dot_annotations__pb2.DESCRIPTOR,])




_QUERYDENOMTRACEREQUEST = _descriptor.Descriptor(
  name='QueryDenomTraceRequest',
  full_name='ibc.applications.transfer.v1.QueryDenomTraceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='hash', full_name='ibc.applications.transfer.v1.QueryDenomTraceRequest.hash', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=215,
  serialized_end=253,
)


_QUERYDENOMTRACERESPONSE = _descriptor.Descriptor(
  name='QueryDenomTraceResponse',
  full_name='ibc.applications.transfer.v1.QueryDenomTraceResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='denom_trace', full_name='ibc.applications.transfer.v1.QueryDenomTraceResponse.denom_trace', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=255,
  serialized_end=343,
)


_QUERYDENOMTRACESREQUEST = _descriptor.Descriptor(
  name='QueryDenomTracesRequest',
  full_name='ibc.applications.transfer.v1.QueryDenomTracesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='pagination', full_name='ibc.applications.transfer.v1.QueryDenomTracesRequest.pagination', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=345,
  serialized_end=430,
)


_QUERYDENOMTRACESRESPONSE = _descriptor.Descriptor(
  name='QueryDenomTracesResponse',
  full_name='ibc.applications.transfer.v1.QueryDenomTracesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='denom_traces', full_name='ibc.applications.transfer.v1.QueryDenomTracesResponse.denom_traces', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\252\337\037\006Traces\310\336\037\000', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pagination', full_name='ibc.applications.transfer.v1.QueryDenomTracesResponse.pagination', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=433,
  serialized_end=600,
)


_QUERYPARAMSREQUEST = _descriptor.Descriptor(
  name='QueryParamsRequest',
  full_name='ibc.applications.transfer.v1.QueryParamsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=602,
  serialized_end=622,
)


_QUERYPARAMSRESPONSE = _descriptor.Descriptor(
  name='QueryParamsResponse',
  full_name='ibc.applications.transfer.v1.QueryParamsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='params', full_name='ibc.applications.transfer.v1.QueryParamsResponse.params', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=624,
  serialized_end=699,
)


_QUERYDENOMHASHREQUEST = _descriptor.Descriptor(
  name='QueryDenomHashRequest',
  full_name='ibc.applications.transfer.v1.QueryDenomHashRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='trace', full_name='ibc.applications.transfer.v1.QueryDenomHashRequest.trace', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=701,
  serialized_end=739,
)


_QUERYDENOMHASHRESPONSE = _descriptor.Descriptor(
  name='QueryDenomHashResponse',
  full_name='ibc.applications.transfer.v1.QueryDenomHashResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='hash', full_name='ibc.applications.transfer.v1.QueryDenomHashResponse.hash', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=741,
  serialized_end=779,
)

_QUERYDENOMTRACERESPONSE.fields_by_name['denom_trace'].message_type = ibc_dot_applications_dot_transfer_dot_v1_dot_transfer__pb2._DENOMTRACE
_QUERYDENOMTRACESREQUEST.fields_by_name['pagination'].message_type = cosmos_dot_base_dot_query_dot_v1beta1_dot_pagination__pb2._PAGEREQUEST
_QUERYDENOMTRACESRESPONSE.fields_by_name['denom_traces'].message_type = ibc_dot_applications_dot_transfer_dot_v1_dot_transfer__pb2._DENOMTRACE
_QUERYDENOMTRACESRESPONSE.fields_by_name['pagination'].message_type = cosmos_dot_base_dot_query_dot_v1beta1_dot_pagination__pb2._PAGERESPONSE
_QUERYPARAMSRESPONSE.fields_by_name['params'].message_type = ibc_dot_applications_dot_transfer_dot_v1_dot_transfer__pb2._PARAMS
DESCRIPTOR.message_types_by_name['QueryDenomTraceRequest'] = _QUERYDENOMTRACEREQUEST
DESCRIPTOR.message_types_by_name['QueryDenomTraceResponse'] = _QUERYDENOMTRACERESPONSE
DESCRIPTOR.message_types_by_name['QueryDenomTracesRequest'] = _QUERYDENOMTRACESREQUEST
DESCRIPTOR.message_types_by_name['QueryDenomTracesResponse'] = _QUERYDENOMTRACESRESPONSE
DESCRIPTOR.message_types_by_name['QueryParamsRequest'] = _QUERYPARAMSREQUEST
DESCRIPTOR.message_types_by_name['QueryParamsResponse'] = _QUERYPARAMSRESPONSE
DESCRIPTOR.message_types_by_name['QueryDenomHashRequest'] = _QUERYDENOMHASHREQUEST
DESCRIPTOR.message_types_by_name['QueryDenomHashResponse'] = _QUERYDENOMHASHRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

QueryDenomTraceRequest = _reflection.GeneratedProtocolMessageType('QueryDenomTraceRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYDENOMTRACEREQUEST,
  '__module__' : 'ibc.applications.transfer.v1.query_pb2'
  # @@protoc_insertion_point(class_scope:ibc.applications.transfer.v1.QueryDenomTraceRequest)
  })
_sym_db.RegisterMessage(QueryDenomTraceRequest)

QueryDenomTraceResponse = _reflection.GeneratedProtocolMessageType('QueryDenomTraceResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYDENOMTRACERESPONSE,
  '__module__' : 'ibc.applications.transfer.v1.query_pb2'
  # @@protoc_insertion_point(class_scope:ibc.applications.transfer.v1.QueryDenomTraceResponse)
  })
_sym_db.RegisterMessage(QueryDenomTraceResponse)

QueryDenomTracesRequest = _reflection.GeneratedProtocolMessageType('QueryDenomTracesRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYDENOMTRACESREQUEST,
  '__module__' : 'ibc.applications.transfer.v1.query_pb2'
  # @@protoc_insertion_point(class_scope:ibc.applications.transfer.v1.QueryDenomTracesRequest)
  })
_sym_db.RegisterMessage(QueryDenomTracesRequest)

QueryDenomTracesResponse = _reflection.GeneratedProtocolMessageType('QueryDenomTracesResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYDENOMTRACESRESPONSE,
  '__module__' : 'ibc.applications.transfer.v1.query_pb2'
  # @@protoc_insertion_point(class_scope:ibc.applications.transfer.v1.QueryDenomTracesResponse)
  })
_sym_db.RegisterMessage(QueryDenomTracesResponse)

QueryParamsRequest = _reflection.GeneratedProtocolMessageType('QueryParamsRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYPARAMSREQUEST,
  '__module__' : 'ibc.applications.transfer.v1.query_pb2'
  # @@protoc_insertion_point(class_scope:ibc.applications.transfer.v1.QueryParamsRequest)
  })
_sym_db.RegisterMessage(QueryParamsRequest)

QueryParamsResponse = _reflection.GeneratedProtocolMessageType('QueryParamsResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYPARAMSRESPONSE,
  '__module__' : 'ibc.applications.transfer.v1.query_pb2'
  # @@protoc_insertion_point(class_scope:ibc.applications.transfer.v1.QueryParamsResponse)
  })
_sym_db.RegisterMessage(QueryParamsResponse)

QueryDenomHashRequest = _reflection.GeneratedProtocolMessageType('QueryDenomHashRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYDENOMHASHREQUEST,
  '__module__' : 'ibc.applications.transfer.v1.query_pb2'
  # @@protoc_insertion_point(class_scope:ibc.applications.transfer.v1.QueryDenomHashRequest)
  })
_sym_db.RegisterMessage(QueryDenomHashRequest)

QueryDenomHashResponse = _reflection.GeneratedProtocolMessageType('QueryDenomHashResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYDENOMHASHRESPONSE,
  '__module__' : 'ibc.applications.transfer.v1.query_pb2'
  # @@protoc_insertion_point(class_scope:ibc.applications.transfer.v1.QueryDenomHashResponse)
  })
_sym_db.RegisterMessage(QueryDenomHashResponse)


DESCRIPTOR._options = None
_QUERYDENOMTRACESRESPONSE.fields_by_name['denom_traces']._options = None

_QUERY = _descriptor.ServiceDescriptor(
  name='Query',
  full_name='ibc.applications.transfer.v1.Query',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=782,
  serialized_end=1458,
  methods=[
  _descriptor.MethodDescriptor(
    name='DenomTrace',
    full_name='ibc.applications.transfer.v1.Query.DenomTrace',
    index=0,
    containing_service=None,
    input_type=_QUERYDENOMTRACEREQUEST,
    output_type=_QUERYDENOMTRACERESPONSE,
    serialized_options=b'\202\323\344\223\002+\022)/ibc/apps/transfer/v1/denom_traces/{hash}',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DenomTraces',
    full_name='ibc.applications.transfer.v1.Query.DenomTraces',
    index=1,
    containing_service=None,
    input_type=_QUERYDENOMTRACESREQUEST,
    output_type=_QUERYDENOMTRACESRESPONSE,
    serialized_options=b'\202\323\344\223\002$\022\"/ibc/apps/transfer/v1/denom_traces',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Params',
    full_name='ibc.applications.transfer.v1.Query.Params',
    index=2,
    containing_service=None,
    input_type=_QUERYPARAMSREQUEST,
    output_type=_QUERYPARAMSRESPONSE,
    serialized_options=b'\202\323\344\223\002\036\022\034/ibc/apps/transfer/v1/params',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DenomHash',
    full_name='ibc.applications.transfer.v1.Query.DenomHash',
    index=3,
    containing_service=None,
    input_type=_QUERYDENOMHASHREQUEST,
    output_type=_QUERYDENOMHASHRESPONSE,
    serialized_options=b'\202\323\344\223\002,\022*/ibc/apps/transfer/v1/denom_hashes/{trace}',
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_QUERY)

DESCRIPTOR.services_by_name['Query'] = _QUERY

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2022 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Tests for BLS KeyPair module of the Crypto Package."""
from cosmpy.crypto.keypairs_bls import (
    PrivateKey,
    PublicKey,
    aggregate_signatures,
    verify_aggregated_signatures,
)


def test_basic_usecase():
    """Test common BLS keys and signatures use case."""
    msg = b"some"
    pk1 = PrivateKey()
    signature1 = pk1.sign(msg)
    assert pk1.verify(msg, signature1)
    pk2 = PrivateKey()
    signature2 = pk2.sign(msg)
    assert pk2.verify(msg, signature2)

    pk3 = PrivateKey()
    signature3 = pk3.sign(msg)
    assert pk3.verify(msg, signature3)

    assert not pk3.verify(msg, signature2)

    aggregated_signature = aggregate_signatures(signature1, signature2)

    assert verify_aggregated_signatures([pk1, pk2], [msg] * 2, aggregated_signature)

    assert not verify_aggregated_signatures([pk1, pk3], [msg] * 2, aggregated_signature)


def test_private_key():
    """Test BLS private key class."""
    priv_key = PrivateKey()
    assert priv_key.private_key
    assert len(priv_key.private_key_bytes) == 32
    assert len(priv_key.private_key_hex) == 64

    new_priv_key = PrivateKey(priv_key.private_key_bytes)
    assert priv_key.public_key_bytes == new_priv_key.public_key_bytes

    message = b"some"
    assert priv_key.sign(message) != priv_key.sign_digest(message)


def test_public_key():
    """Test BLS public key class."""
    priv_key = PrivateKey()
    pub_key = PublicKey(priv_key.public_key_bytes)
    assert pub_key.public_key
    assert len(pub_key.public_key_bytes) == 48
    assert len(pub_key.public_key_hex) == 96

    message = b"some"
    signature = priv_key.sign(message)
    signature_for_digest = priv_key.sign_digest(message)

    assert pub_key.verify(message, signature)
    assert pub_key.verify_digest(message, signature_for_digest)

    assert not pub_key.verify(message, signature_for_digest)
    assert not pub_key.verify_digest(message, signature)


def test_pub_key_hardcoded_sgianture_check():
    """Check signature of the pub key provided."""
    pub_key = PublicKey(
        b"\x92\x08eo\xb9\x03\xf1\\\x05\xd4\x8fa\xfd}-\x913\\\x1e~\xe6\xbd\xd3Eu\xf43\x05\x88\xfc\x1f\x18\x08\xf8B\x98\xf6-v\xc0\xcbQvpb\xcf\xe0q"
    )
    message = b"random message"
    signature = b"\x87\xcd\xf8z\xfe|P\xe6\x93\x15H\xdd\x9e&\xbf(\xaa\x06\xb7\xce3\xba\x1e \x0e\xea3\x8f3\xdc$.\x04V\xa9,\xd7\xa1\xf5^\x13\x13\xfeNd\xa8\xcb,\x14\xc3p|Zy\xe8\x95\xf5'%Q\x82\xc9,\xe4\x88x\xae\xd3?\xd0\\D]\xcfQ:\xf6\xb9\xdbK\xf5\xbeS\x16\x05\x06\x15\xfff\x1b\x0f\xf4\x1a\xc4\xeb\xf0"

    assert pub_key.verify(message, signature)


def test_aggregated_signature_hardcoded():
    """Check hardcoded aggragated signature."""
    msg = b"some"
    pub_key1 = PublicKey(
        b"\x84}\x9d\xa2\x90\xa78%?\xb9\x1c\x86/:4\xa4b\x12\x95\xff1\xde\xc6L'P\xd9\x8e\xd0\x0b \x7f \xcd\xfc\xeeO'\xc8d\xbd\x06\x9d\xbb\xb2\xcd\x16R"
    )
    pub_key2 = PublicKey(
        b"\xac\xa7\xf3\x02JSsv\xbf*f\xde\x8c\xec\xa1\x83|\x84\xcc\tU2\xfb /\xce\xb8\xd3$hY\r\xbf\x99/\xa1\xccU\x12\x027z\x1a\xa549\x88\\"
    )
    aggregated_signature = b"\x93\xd9\x80l3[\xaa\xb3\xfa|\xf0\x84\x04O\xac\xe7H\xd6\xa7\xbc \t\x8bB\xdfO\x886\xdbW\xda5F \xfb\x91\xd9\x85x \xab\xeb\xb3\xb3\x10\xc0$M\x02\xcf\x06\xb0L|W\xbb\x1a\xed0\xf6\x17[\xd8\x01\xb5\x96\xf3\xe5\xfbC\x17'\xaf\xe5\xa5\x8e\xc1\xbe\xa1) \xac\x8cC\x1f\xbd\xb8\xfb\x9b\x14!g\xbec\r>"

    assert verify_aggregated_signatures(
        [pub_key1, pub_key2], [msg] * 2, aggregated_signature
    )
